<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class supports Services
 *
 * @package    Service
 * @category   Models
 * @author     Deon George
 * @copyright  (c) 2009-2013 Open Source Billing
 * @license    http://dev.osbill.net/license.html
 *
 * Fields:
 * + queue: PROVISION (to be provisioned)
 */
class Model_Service extends ORM_OSB {
	// Relationships
	protected $_has_one = array(
		'service_billing'=>array('far_key'=>'account_billing_id','foreign_key'=>'id'),
	);
	protected $_has_many = array(
		'invoice_item'=>array('far_key'=>'id'),
		'invoice'=>array('through'=>'invoice_item'),
		'service_change'=>array('far_key'=>'id'),
	);
	protected $_belongs_to = array(
		'product'=>array(),
		'account'=>array(),
	);

	/**
	 * Filters used to format the display of values into friendlier values
	 */
	protected $_display_filters = array(
		'date_last_invoice'=>array(
			array('Config::date',array(':value')),
		),
		'date_next_invoice'=>array(
			array('Config::date',array(':value')),
		),
		'recur_schedule'=>array(
			array('StaticList_RecurSchedule::get',array(':value')),
		),
		'status'=>array(
			array('StaticList_YesNo::get',array(':value')),
		),
	);

	/**
	 * Return the object of the product plugin
	 */
	public function plugin($type='') {
		if (! $this->product->prod_plugin_file)
			return NULL;

		$o = ORM::factory(Kohana::classname(sprintf('Service_Plugin_%s',$this->product->prod_plugin_file)),array('service_id'=>$this->id));

		return $type ? $o->$type : $o;
	}

	/**
	 * Display the service number
	 */
	public function id() {
		return sprintf('%05s',$this->id);
	}

	/**
	 * Display the service product name
	 */
	public function name() {
		return is_null($plugin=$this->plugin()) ? $this->product->title() : $plugin->name();
	}

	public function pending_change() {
		return count($this->service_change->where_active()->where_open()->and_where('complete','!=',1)->or_where('complete','IS',null)->where_close()->find_all()->as_array());
	}

	/**
	 * Display how much is due on this service
	 */
	public function due($format=FALSE) {
		$total = 0;

		foreach ($this->list_invoices(TRUE) as $io)
			$total += $io->due();

		return $format ? Currency::display($total) : $total;
	}

	/**
	 * When does this service expire
	 */
	public function expire($format=FALSE) {
		// For plugins the plugin determins expiry
		$expire = (is_null($plugin=$this->plugin()) ? NULL : $plugin->expire());

		// If $expire is NULL, we'll use the next invoice date
		$expire = is_null($expire) ? $this->date_next_invoice-86400 : $expire;

		return $format ? Config::date($expire) : $expire;
	}

	/**
	 * Determine if a service expires in the next $days.
	 */
	public function expiring($days) {
		return time()+$days*86400 > $this->expire();
	}

	/**
	 * Return the service charge
	 */
	public function price($tax=FALSE,$format=FALSE) {
		$x = $this->product->keyget('price_group',$this->recur_schedule);

		// @todo This index shouldnt be hard coded.
		$p = $this->price ? $this->price : $x[$this->price_group]['price_base'];

		if ($tax)
			$p = Tax::add($p);

		return $format ? Currency::display($p) : $p;
	}

	public function service_name() {
		return is_null($plugin=$this->plugin()) ? $this->name() : $plugin->service_name();
	}

	public function service_view() {
		return is_null($plugin=$this->plugin()) ? HTML::nbsp('') : $plugin->service_view();
	}

	/**
	 * Display the product feature summary
	 */
	public function product_feature_summary() {
		return $this->product->feature_summary();
	}

	/**
	 * Render some details for specific calls, eg: invoice
	 */
	public function details($type) {
		switch ($type) {
			case 'invoice_detail_items':
				if (is_null($plugin = $this->plugin()))
					return array();
				else
					return $plugin->_details($type);
				break;

			default:
				throw new Kohana_Exception('Unkown detail request :type',array(':type'=>$type));
		}
	}

	/**
	 * Enable the plugin to store data
	 */
	public function admin_update() {
		if (is_null($plugin = $this->plugin()))
			return NULL;
		else
			return $plugin->admin_update();
	}

	public function transactions() {
		return $this->invoice_item->order_by('date_start,date_stop');
	}

	// @todo To implement
	public function charges_new() {
		return 0;
	}

	/** LIST FUNCTIONS **/

	/**
	 * Search for services matching a term
	 */
	public function list_autocomplete($term,$index='id',array $limit=array()) {
		$result = array();

		$this->clear();
		$this->where_active();
		$value = 'service_name()';

		// Build our where clause
		$this->where_open()
				->where('id','like','%'.$term.'%')
				->where_close();

		foreach ($limit as $w) {
			list($k,$s,$v) = $w;

			$this->and_where($k,$s,$v);
		}

		foreach ($this->find_all() as $o)
			$result[$o->$index] = array(
				'value'=>$o->$index,
				'label'=>sprintf('SVC %s: %s',$o->id,Table::resolve($o,$value)),
			);

		return $result;
	}
	
	public function list_bylistgroup($cat) {
		$result = array();

		foreach ($this->list_active() as $so)
			if ($so->product->prod_plugin_file == $cat)
				array_push($result,$so);

		Sort::MASort($result,'service_name()');
		return $result;
	}

	/**
	 * List services expiring
	 */
	public function list_expiring($days=14) {
		$result = array();

		foreach ($this->list_active() as $so)
			if ($so->expiring($days))
				array_push($result,$so);

		return $result;
	}

	/**
	 * List invoices for this service
	 */
	public function list_invoices($due=FALSE) {
		$result = array();

		foreach ($this->invoice->find_all() as $io)
			if (! $due OR $io->due())
				array_push($result,$io);

		return $result;
	}

	/**
	 * List services that need to be billed.
	 *
	 * @param $days int Additional number of days to add to the query, above the module config.
	 */
	public function list_invoicesoon($days=0) {
		return $this->_where_active()
			->where_open()->where('suspend_billing','IS',NULL)->or_where('suspend_billing','=','0')->where_close()
			->where('date_next_invoice','<',time()+(ORM::factory('Invoice')->config('GEN_DAYS')+$days)*86400)
			->find_all();
	}

	/**
	 * List services that need to be provisioned
	 */
	public function list_provision() {
		return $this->_where_active()->where('queue','=','PROVISION');
	}
}
?>
